@@ -1,4 +1,4 @@
-svsim <- function(len, mu = -10, phi = 0.98, sigma = 0.2) {
+svsim <- function(len, mu = -10, phi = 0.98, sigma = 0.2, nu = Inf) {
  
  # Some error checking
  if (any(is.na(len)) | !is.numeric(len) | length(len) != 1 | any(len < 1)) {
@@ -19,19 +19,28 @@ if (!is.numeric(sigma) | length(sigma) != 1 | sigma <= 0) {
   stop("Argument 'sigma' (volatility of latent variable) must be a single number > 0.")
  }
 
+if (!is.numeric(nu) || length(nu) != 1 || nu <= 2) {
+ stop("Argument 'nu' (degrees of freedom for the conditional error) must be a single number > 2.")
+}
+
  h <- rep(as.numeric(NA), len)
  h0 <- rnorm(1, mean=mu, sd=sigma/sqrt(1-phi^2))
- nu <- rnorm(len)
+ innov <- rnorm(len)
 
  # simulate w/ simple loop
- h[1] <- mu + phi*(h0-mu) + sigma*nu[1]
- for (i in seq(2, len = len-1)) h[i] <- mu + phi*(h[i-1]-mu) + sigma*nu[i]
+ h[1] <- mu + phi*(h0-mu) + sigma*innov[1]
+ for (i in seq(2, len = len-1)) h[i] <- mu + phi*(h[i-1]-mu) + sigma*innov[i]
 
- y <- rnorm(len, mean=0, sd=sqrt(exp(h)))  # "log-returns"
+ if (is.infinite(nu)) {
+  y <- exp(h / 2) * rnorm(len)  # "log-returns"
+ } else {
+  y <- exp(h / 2) * rt(len, df = nu)  # "log-returns"
+ }
  ret <- list(y = y, vol = exp(h/2), vol0 = exp(h0/2),
 	     para = list(mu = mu,
 			 phi = phi,
 			 sigma = sigma))
+ if (is.finite(nu)) ret$para$nu <- nu
  class(ret) <- "svsim"
  ret
 }
@@ -39,11 +48,13 @@ if (!is.numeric(sigma) | length(sigma) != 1 | sigma <= 0) {
 print.svsim <- function(x, ...) {
  cat("\nSimulated time series consisting of", length(x$y), "observations.\n
 Parameters: level of latent variable                  mu =", x$para$mu, "
-	    persistence of latent variable           phi =", x$para$phi, "
-	    standard deviation of latent variable  sigma =", x$para$sigma, "
+            persistence of latent variable           phi =", x$para$phi, "
+            standard deviation of latent variable  sigma =", x$para$sigma, "
+            ")
+ if (length(x$para) == 4) cat("degrees of freedom parameter              nu =", x$para$nu, "
 	    ")
- cat("\nSimulated initial volatility:", x$vol0, "\n")
- cat("\nSimulated volatilities:\n")
+ cat("\nSimulated initial conditional volatility:", x$vol0, "\n")
+ cat("\nSimulated conditional volatilities:\n")
  print(x$vol, ...)
  cat("\nSimulated data (usually interpreted as 'log-returns'):\n")
  print(x$y, ...)
@@ -54,7 +65,7 @@ plot.svsim <- function(x, mar = c(3, 2, 2, 1), mgp = c(1.8, .6, 0), ...) {
  plot.ts(100*x$y, ylab = "", ...)
  mtext("Simulated data: 'log-returns' (in %)", cex = 1.2, line = .4, font = 2)
  plot.ts(100*x$vol, ylab = "", ...)
- mtext("Simulated volatilities (in %)", cex = 1.2, line = .4, font = 2)
+ mtext("Simulated conditional volatilities (in %)", cex = 1.2, line = .4, font = 2)
  par(op)
 }
 
@@ -77,10 +88,12 @@ print.summary.svsim  <- function(x, ...) {
      x$para$phi,
      "\n            standard deviation of latent variable  sigma = ",
      x$para$sigma, "\n", sep="")
+ if (length(x$para) == 4) cat("            degrees of freedom parameter              nu =", x$para$nu, "
+	    ")
 
- cat("\nSimulated initial volatility (in %): ")
+ cat("\nSimulated initial conditional volatility (in %): ")
  cat(x$vol0, "\n")
- cat("\nSummary of simulated volatilities (in %):\n")
+ cat("\nSummary of simulated conditional volatilities (in %):\n")
  print(x$vol)
  cat("\nSummary of simulated data (in %):\n")
  print(x$y)
